/***************************************************************************
 *   Copyright (C) 2007 by Gustavo Pichorim Boiko                          *
 *   boiko@mandriva.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef PANEL_H
#define PANEL_H

#include <qwidget.h>
#include <qpixmap.h>
#include <qvariant.h>
#include <qwmatrix.h>
#include "selection.h"

class KRootPixmap;
class PanelItem;
class KConfig;
class KPopupMenu;
class PanelManager;

typedef QValueList<PanelItem*> ItemList;

/**
 * The panel class implements all the necessary bits for a generic panel.
 * If you want to create a new panel type, you should inherit this class.
 * Panels are transparent widgets shown in the desktop containing different
 * kinds of items. 
 * 
 * A base class for panel items is implemented in the \ref PanelItem class.
 * For a simple example of a reimplemented panel, see the \ref ActionPanel class.
 * For a more complete example, see the \ref ServicePanel class.
 *
 * When implementing a new panel type, you should also implement a function for 
 * creating this panel in \ref PanelManager and also adjust 
 * \ref PanelManager::readConfig() and add an entry there for your panel.
 * The reimplemented panel should implement \ref type() returning an string
 * identifying the panel type.
 */
class Panel : public QWidget
{
Q_OBJECT
public:
    /**
     * Creates a new panel.
     *  @param id The panel ID. This should be a uniq identifier for the panel
     *  @param argb This tells the panel wheter to use a real trasparent background
     *              or a fake background using \ref KRootPixmap
     *  @param parent A pointer to the \ref PanelManager instance that is managing
     *                this panel.
     *  @param name The name of this panel. See \ref QWidget::name()
     */ 
    Panel(int id, bool argb, PanelManager *parent = 0, const char *name = 0);

    ~Panel();
    
    /**
     * Sets the top image of the panel
     *  @param top The top pixmap
     * 
     * @see \ref setBottom(), \ref setCenter()
     */
    void setTop(const QPixmap &top);
    
    /**
     * Sets the bottom image of the panel
     *  @param bottom The bottom pixmap
     *
     * @see \ref setTop(), \ref setCenter()
     */
    void setBottom(const QPixmap &bottom);
    
    /**
     * Sets the image that will be used under the panel items
     *  @param center The center pixmap. It is a good idea to set an image in 
     *                that the first line of the image matches the last line, 
     *                as this image is used multiple times according to the 
     *                number of items in the panel.
     *
     *  @see \ref setTop(), \ref setBottom()
     */
    void setCenter(const QPixmap &center);
    
    /**
     * Sets a color for the panel
     * The given color will be applied to all pixmaps
     *  @param color The color to be applied
     * 
     * @see \ref setTop(), \ref setBottom(), \ref setCenter()
     */ 
    void setColor(const QColor &color);
    
    /**
     * Returns the current color of the panel
     * @return The color of the panel
     */
     QColor color() const;
     
    /**
     * Sets a title for the panel.
     * In the default implementation the title will appear in the top of the 
     * panel
     *  @param title The title string. This string should not be long or it
     *               will be truncated
     */
    void setTitle(const QString &title);
    QString title() const;
    
    /**
     * Triggers an update in the panel. Updates are usually necessary when the item
     * contents has changed
     */
    void refresh();
    
    /**
     * Adds an item to the panel. This function is usually called from the 
     * derived panels after they have created the item.
     *  @param item The item to be added
     *  @param update Tells wheter the panel should be updated or not. This is
     *                useful when adding multiple items to the panel.
     */
    void addItem(PanelItem *item, bool update = true);
    
    /**
     * Removes an item from the panel.
     *  @param item The item to be removed
     *  @param update Tells wheter the panel should be updated or not. This is
     *                useful when adding multiple items to the panel.
     */ 
    void removeItem(PanelItem *item, bool update = true);
    
    /**
     * Removes all items from the panel
     *  @param update Tells wheter the panel should be updated or not. This is
     *                useful when updating the panel contents. In this case you
     *                only wants the panel to be updated after the new items 
     *                have been added.
     */
    void clear(bool update = true);
    
    /**
     * The list of items
     *  @return This function returns a const representation of the list of the 
     *          items.
     */
    ItemList items() const;
    
    /**
     * The id of the panel
     * @return The panel id
     */
    int id();
    
    /**
     * The type of the panel. This should be reimplemented in derived classes
     * @return A string representing the panel type
     */
    virtual QString type() const { return "Default"; }
    
    /* scale stuff */
    /**
     * Sets the scale to be used when drawing and calculating coordinates
     * This is mostly used by the \ref PanelManager to adjust the panel size
     * to the screen resolution.
     *  @param scaleX The scale to be used in the X axis
     *  @param scaleY The scale to be used in the Y axis
     */
    void setScale(double scaleX, double scaleY);
    
    /**
     * Converts a point from the base resolution coordinations to the real ones
     *  @param p The point in base resolution coordinates
     */
    QPoint fromBase(const QPoint &p);
    
    /**
     * Converts a point from the real widget coordinations to the base resolution
     * ones.
     *  @param p The point in real widget coordinations
     */
    QPoint toBase(const QPoint &p);
    
    /**
     * Converts a size from the base resolution coordinations to the real ones
     *  @param s The size in base resolution coordinates
     */
    QSize fromBase(const QSize &s);
    
    /**
     * Converts a size from the real widget coordinations to the base resolution
     * ones.
     *  @param s The size in real widget coordinations
     */
    QSize toBase(const QSize &s);
    
    /**
     * Converts a rect from the base resolution coordinations to the real ones
     *  @param r The rect in base resolution coordinates
     */
    QRect fromBase(const QRect &r);
    
    /**
     * Converts a rect from the real widget coordinations to the base resolution
     * ones.
     *  @param r The rect in real widget coordinations
     */
    QRect toBase(const QRect &r);
    
    /* config stuff */
    /**
     * Read panel specific configuration. This can be reimplemented in the derived
     * classes to read specific configuration.
     *  @param cfg A \ref KConfig instance already pointing to the panel 
     *             configuration group.
     */
    virtual void readCfg(KConfig *cfg);
    
    /**
     * Loads an icon to be used in the context menu. This is here just for convenience
     *  @param icon The name of the icon to be loaded
     */
    static QPixmap loadIcon(const QString &icon);
    
    /**
     * Write the configuration entries
     */
    void writeConfig();
    
    void setTitleFont(const QFont &font);
    QFont titleFont() const;
    
    void setItemFont(const QFont &font);
    QFont itemFont() const;
    
    bool useShadow() const;
    void setUseShadow(bool shadow);
    
    QColor textColor() const;
    void setTextColor(const QColor &color);
    
    /**
     * This property is used by the PanelManager when the configuration changes
     * and the panel type changes from one type to another. Then the previous panel
     * should be deleted and a new one should be created.
     * Do NOT change the value of this variable from inside the \ref Panel class.
     */
    bool can_delete;
    
signals:
    /**
     * This signal gets emitted when the user changes the position of the panel in the screen.
     * The \ref PanelManager is responsible by writing the new position to the config file as
     * it needs to converts the coordinates to the base resolution ones.
     */
    void positionChanged(Panel *pan);
    
public slots:
    /**
     * Read the configuration. This will also trigger panel specific settings reading
     */
    void readConfig();
    
    /**
     * Writes a config entry for this panel
     *  @param entry The entry name
     *  @param value The value for the entry
     */
    void writeConfigEntry(const QString &entry, const QVariant &value);
    
    /**
     * Toggle locking of the panel position.
     */
    void toggleLocked();
    void setLocked(bool l);
    bool locked() const;
    
    /**
     * Opens a color dialog asking to change the color
     */
    void slotChangeColor();
    
    /**
     * Opens a edit dialog asking to change the title
     */
    void slotChangeTitle();
    
    /**
     * Shows the configure dialog
     */
    void slotConfigure();
    
protected:
    /* mouse handling stuff */
    virtual void mousePressEvent(QMouseEvent *e);
    virtual void mouseReleaseEvent(QMouseEvent *e);
    virtual void mouseDoubleClickEvent(QMouseEvent *e);
    virtual void mouseMoveEvent(QMouseEvent *e);
    
    /* drag and dro handling stuff */
    virtual void dragEnterEvent(QDragEnterEvent *event);
    virtual void dragMoveEvent(QDragMoveEvent *event);
    virtual void dragLeaveEvent(QDragLeaveEvent *event);
    virtual void dropEvent(QDropEvent *event);
    
    /**
     * Checks wheter a drop event can be handled by a panel
     * The panel can reimplement this when it accepts drops
     *  @param event The event
     */
    virtual bool canHandle(QDropEvent *event);
    
    /**
     * Handle a drop event.
     * The default implementation does not handle any drop events so this 
     * should be reimplemented in derived panels
     *  @param event The drop event
     */
    virtual bool handleDrop(QDropEvent *event);
    
    /* widget stuff */
    virtual void paintEvent(QPaintEvent *event);
    virtual void contextMenuEvent(QContextMenuEvent *event);
    
    /**
     * Put panel specific options in the popup menu. The default implementation
     * does not add any entry to the menu. This can be reimplemented in derived
     * panels to add specific entries.
     *  @param menu The popup menu
     */
    virtual void panelMenu(KPopupMenu *menu);
    
    /**
     * Updates the size of the panel and triggers a redraw in the panel
     */
    void updateSize();

private:
    PanelManager *m_parent;
    ItemList m_itemList;
    bool m_useArgb;
    KRootPixmap *m_rootPixmap;
    QPixmap m_bg;
    QPixmap m_top;
    QPixmap m_bottom;
    QPixmap m_center;
    QColor m_color;
    int m_id;
    PanelItem *m_pressedItem;
    bool m_moving;
    QPoint m_clickPos;
    QPoint m_pos; 
    bool m_locked;
    QString m_title;
    Selection m_selection;
    QFont m_titleFont;
    QFont m_itemFont;
    bool m_useShadow;
    QColor m_textColor;
    
    /* scale stuff*/
    QWMatrix m_matrix;
    QWMatrix m_invMatrix;
    
    void renderBackground();
    void applyColor(QPixmap &pix, bool reset = false);
    PanelItem *itemAt(QPoint pos);
    
private slots:
    void slotUpdateBackground(const QPixmap &pix);
};

#endif
