/***************************************************************************
 *   Copyright (C) 2007 by Gustavo Pichorim Boiko                          *
 *   boiko@mandriva.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef PANELITEM_H
#define PANELITEM_H

#include <qobject.h>
#include <qpixmap.h>
#include <qstringlist.h>

class Panel;
class KPopupMenu;

/**
 * This class represents an item of a panel. This is a base class and to get 
 * more useful items, this class should be reimplemented.
 * For an example of a simple reimplemented item, see the \ref ActionItem class.
 * For a more complete example, see the \ref ServiceItem class.
 */
class PanelItem : public QObject
{
Q_OBJECT
public:
    /**
     * The constructor of the item
     *  @param pan The panel this item belongs to
     */
    PanelItem(Panel *pan);

    ~PanelItem();
    
    /**
     * The text of the item
     */
    virtual QString text() const;
    /**
     * The icon that appears in the left of the item
     */
    virtual QPixmap icon() const;
    /**
     * Paint the button using the given painter and the bounding rect
     *  @param p The painter to be used
     *  @param r The rect that this item should use
     */
    virtual void paint(QPainter *p, QRect r);
    
    /**
     * Tells wheter an item is removable. This function can be reimplemented in
     * derived classes that have read-only items.
     * @return Returns true if the item is removable, or false if it is read-only
     */ 
    virtual bool canRemove();
    
    /**
     * The panel this item belongs to
     * @return Returns a pointer to the panel
     */
    Panel *panel();
    
    /**
     * Triggers an update in the parent panel
     */
    void refreshPanel();
    
    /**
     * Checks wheter a point is over this item
     *  @param pos The poing to be verified
     *
     * @return Returns true when the point is over this item, and false when it
     *         is not
     */
    bool pointIsOver(QPoint pos);
    
    /* for drag and drop */
    /**
     * Checks wheter this item can handle a drop event. The base item does not
     * handle any drop event, so if a panel item supports drop, this function
     * needs to be reimplemented.
     *  @param event The drop event
     * 
     * @return Returs true if the item can handle the drop event, false when it
     *         can't
     */
    virtual bool canHandle(QDropEvent *event);
    
    /**
     * Handle a drop event. The base implementation does not handle drop events
     * at all, so if a derived item supports drop, this should be reimplemented.
     *  @param event The drop event
     * 
     * @return Returns true when the event was handle successfully, false when it
     *         was not.
     */
    virtual bool handleDrop(QDropEvent *event);
    
    /**
     * This trigger an item removal
     * Further implementations should explicitly call 
     * PanelItem::remove();
     */
    virtual void remove();
    
    /**
     * Adds item specific options to the popup menu.
     * The default implementation doesn't add any option to the menu, so if you
     * want to add options to derived panel items, you should reimplement this.
     *  @param menu The popup menu
     */
    void itemMenu(KPopupMenu *menu);

    /**
     * The last rect used for this item
     * Rects for each button can change when items are added or removed
     * @returns The rect
     */
     QRect rect() const;
     
public slots:
    /**
     * Triggers an item removal. This is used mainly by the context menu.
     */
    void slotRemove();
    
    /**
     * This function is called when an item is clicked or when a drop event
     * occurs if the panel supports drop.
     * The default implementation does nothing, so this should be reimplemented
     * in derived classes.
     *  @param args The arguments to be used when launching the item.
     */ 
    virtual bool launch(const QStringList &args = QStringList());
    
protected:
    /**
     * Sets the text to be displayed in this item.
     *  @param txt The text to be set
     */
    void setText(const QString& txt);
    
    /**
     * Sets the icon for this item
     *  @param ico The icon
     */
    void setIcon(const QString& ico);
    
    /**
     * This is a covenience function to load icons to be displayed in this item.
     * It uses \ref KIconLoader to load the icons.
     *  @param ico The name of the icon
     *
     * @return Returns the loaded icon
     */
    QPixmap loadIcon(const QString &ico); 
    
    void setBaseIconSize(int size);
private:
    QString m_text;
    QPixmap m_icon;
    QString m_iconName;
    int m_baseIconSize;
    int m_iconSize;
    Panel *m_panel;
    QRect m_rect;
    
};

#endif
