/***************************************************************************
 *   Copyright (C) 2007 by Gustavo Pichorim Boiko                          *
 *   boiko@mandriva.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "panelmanager.h"
#include "servicepanel.h"
#include "actionpanel.h"
#include "dirpanel.h"
#include "recentpanel.h"

#include <kglobal.h>
#include <kstandarddirs.h>
#include <kdebug.h>
#include <kdirwatch.h>
#include <qdesktopwidget.h>
#include <qapplication.h>

#define BASE_WIDTH 1024.
#define BASE_HEIGHT 768.

PanelManager::PanelManager(bool use_argb, QObject *parent, const char *name)
 : QObject(parent, name)
{
    m_useArgb = use_argb;
    m_theme = "default";
    m_scaleX = 1.0;
    m_scaleY = 1.0;
    m_invScaleX = 1.0;
    m_invScaleY = 1.0;
    
    slotScreenResized();
    
    connect(QApplication::desktop(), SIGNAL(resized(int)), 
            this, SLOT(slotScreenResized()));

    QStringList configs = KGlobal::dirs()->findDirs("config", ""); 
    
    for (unsigned int i=0; i < configs.count(); ++i)
    {
        KDirWatch::self()->addFile(configs[i] + "kpanelsrc");
    }
    
    connect(KDirWatch::self(), SIGNAL(dirty(const QString&)),
            this, SLOT(readConfig()));
    connect(KDirWatch::self(), SIGNAL(created(const QString&)),
            this, SLOT(readConfig()));
    connect(KDirWatch::self(), SIGNAL(deleted(const QString&)),
            this, SLOT(readConfig()));
            
    KDirWatch::self()->startScan(false, false);        
    readConfig();
}


PanelManager::~PanelManager()
{
}

void PanelManager::createServicePanel(int id)
{
    addPanel(new ServicePanel(id, m_useArgb, this));
}

void PanelManager::createActionPanel(int id)
{
    addPanel(new ActionPanel(id, m_useArgb, this));
}

void PanelManager::createDirPanel(int id)
{
    addPanel(new DirPanel(id, m_useArgb, this));
}

void PanelManager::createRecentPanel(int id)
{
    addPanel(new RecentPanel(id, m_useArgb, this));
}

void PanelManager::applyTheme(Panel *pan)
{
    QString themeBase;
    QPixmap panelTop, panelBottom, panelCenter, actionTop, actionBottom, actionCenter;
    KStandardDirs *dirs = KGlobal::dirs();
    
    themeBase = "themes/" + m_theme + "/";
    
    panelTop.load(dirs->findResource("appdata", themeBase + "panel_top.png"));
    panelBottom.load(dirs->findResource("appdata", themeBase + "panel_bottom.png"));
    panelCenter.load(dirs->findResource("appdata", themeBase + "panel_center.png"));
    
    actionTop.load(dirs->findResource("appdata", themeBase + "action_top.png"));
    actionBottom.load(dirs->findResource("appdata", themeBase + "action_bottom.png"));
    actionCenter.load(dirs->findResource("appdata", themeBase + "action_center.png"));
    
    if (pan)
    {
        if (pan->type() == "Action")
        {
            pan->setTop(actionTop);
            pan->setBottom(actionBottom);
            pan->setCenter(actionCenter);
        }
        else
        {
            pan->setTop(panelTop);
            pan->setBottom(panelBottom);
            pan->setCenter(panelCenter);
        }
        return;
    }
    
    for (unsigned int i = 0; i < m_panelList.count(); ++i)
    {
        Panel *p = m_panelList[i];
        if (p->type() == "Action")
        {
            p->setTop(actionTop);
            p->setBottom(actionBottom);
            p->setCenter(actionCenter);
        }
        else
        {
            p->setTop(panelTop);
            p->setBottom(panelBottom);
            p->setCenter(panelCenter);
        }
    }
}

void PanelManager::setTheme(const QString &theme)
{
    m_theme = theme;
    applyTheme();
}

void PanelManager::readConfig()
{
    KConfig cfg("kpanelsrc");
    
    cfg.setGroup("General");
    setTheme(cfg.readEntry("Theme", "default"));
    int numPanels = cfg.readNumEntry("NumPanels", 3);
    
    /* 
     * Mark all panels as removable. The ones that are not referenced in the new
     * new config will be deleted later
     */ 
    for (unsigned int i=0; i < m_panelList.count(); ++i)
        m_panelList[i]->can_delete = true;
        
    QString type;
    for (int i=0; i < numPanels; ++i)
    {
        cfg.setGroup("Panel_" + QString::number(i));
        type = cfg.readEntry("Type", "Service");
        
        Panel *pan = panelById(i);
        if (pan)
        {
            if (pan->type() != type)
            {
                deletePanel(pan);
                pan = NULL;
            }
            else
                pan->can_delete = false;
        }
        if (!pan)
        {
            if (type == "Service")
                createServicePanel(i);
            else if (type == "Action")
                createActionPanel(i);
            else if (type == "Dir")
                createDirPanel(i);
            else if (type == "Recent")
                createRecentPanel(i);
        }
    }
    
    //now call the config readers
    // and delete the obsolete panels
    Panel *pan;
    for (int i=m_panelList.count()-1; i >=0; --i)
    {
        pan = m_panelList[i];
        if (pan->can_delete)
            deletePanel(pan);
        else
        {
            pan->readConfig();
            pan->show();
        }
    }
    applyTheme();
    adjustPanelPositions();
}

Panel *PanelManager::panelById(int id)
{
    Panel *pan;
    for (unsigned int i=0; i < m_panelList.count(); ++i)
    {
        pan = m_panelList[i];
        if (pan->id() == id)
           return pan;
    }
    return NULL;
}

void PanelManager::deletePanel(Panel *pan)
{
    PanelList::iterator it = m_panelList.find(pan);
    
    if (it != m_panelList.end())
        m_panelList.remove(it);
    delete pan;
}

void PanelManager::slotScreenResized()
{
    QDesktopWidget w;
    m_geometry = w.screenGeometry();
    kdDebug() << "Current screen size: " << m_geometry << endl;
    kdDebug() << "NumScreens: " << w.numScreens() << endl;
    
    m_scaleX = m_geometry.width() / BASE_WIDTH;
    m_scaleY = m_geometry.height() / BASE_HEIGHT;
    m_invScaleX = 1. / m_scaleX;
    m_invScaleY = 1. / m_scaleY;
    
    for (unsigned int i=0; i < m_panelList.count(); ++i)
        m_panelList[i]->setScale(m_scaleY, m_scaleY);
        
    adjustPanelPositions();
}

void PanelManager::addPanel(Panel *pan)
{
    applyTheme(pan);
    m_panelList.append(pan);
    pan->setScale(m_scaleY, m_scaleY);
    
    connect(pan, SIGNAL(positionChanged(Panel*)),
            this, SLOT(slotPositionChanged(Panel*)));
}

void PanelManager::adjustPanelPositions()
{
    Panel *pan;
    KConfig cfg("kpanelsrc", true);
    QPoint pos;
    
    for (unsigned int i=0; i < m_panelList.count(); ++i)
    {
        pan = m_panelList[i];
        cfg.setGroup("Panel_" + QString::number(pan->id()));
        pos = cfg.readPointEntry("Position", &pos);
        pos.setX((int)(pos.x() * m_scaleX));
        pos.setY((int)(pos.y() * m_scaleY));
        pan->move(pos);
    }
}

void PanelManager::slotPositionChanged(Panel * pan)
{
    
    KDirWatch::self()->stopScan();
    KConfig cfg("kpanelsrc");
    cfg.setGroup("Panel_" + QString::number(pan->id()));
    QPoint position((int)(pan->x() * m_invScaleX), (int)(pan->y() * m_invScaleY));
    cfg.writeEntry("Position", position);
    cfg.sync();
    KDirWatch::self()->startScan(false, false);
}

#include "panelmanager.moc"
