/***************************************************************************
 *   Copyright (C) 2007 by Gustavo Pichorim Boiko                          *
 *   boiko@mandriva.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef PANELMANAGER_H
#define PANELMANAGER_H

#include <qobject.h>
#include <qrect.h>

class Panel;

typedef QValueList<Panel*> PanelList;

/**
 * This is the main class of KPanels. 
 * It is in charge of managing the panels and its positioning on the screen.
 * In order to handle screen size changes, it saves the panel position in a
 * base screen resolution, and the position for other sizes are calculated
 * in relation to it.
 */
class PanelManager : public QObject
{
Q_OBJECT
public:
    /**
     * The construtor of the Panel Manager.
     *  @param use_argb Tells wheter the panels should use true transparency or
     *                  fake transparency using \ref KRootPixmap
     *  @param parent The parent of this object. See \ref QObject::parent() for
     *                more information
     *  @param name The name of the object. See \ref QObject::name() for more
     *              information
     */
    PanelManager(bool use_argb, QObject *parent = 0, const char *name = 0);

    ~PanelManager();
    
    /**
     * Creates a new service application panel.
     * Service panels are the ones that contains application icons
     *  @param id The ID for the new panel
     */
     void createServicePanel(int id);
     
     /**
      * Creates a new dir panel.
      * Dir panels are the ones able to list any kind of directory
      * KIO supports.
      *  @param id The ID for the new panel
      */
      void createDirPanel(int id);
      
     /**
      * Creates a new action panel.
      * Action panels are the ones that handle actions like Lock, Logout, etc
      *  @param id The ID for the new panel
      */
      void createActionPanel(int id);
      
      /**
      * Creates a new recent documents panel.
      * Recent Documents Panels are the ones that show documents recently used inside KDE
      *  @param id The ID for the new panel
      */
      void createRecentPanel(int id);
      
      /**
       * Apply a theme to the panels
       *  @param pan The panel that will have the theme applied. If left null, apply to all
       */
      void applyTheme(Panel *pan = NULL);
      
      /**
       * Configure a new theme for kpanels
       *  @param theme the name of the theme
       */
      void setTheme(const QString &theme);
      
      /**
       * Finds a panel by its ID.
       *  @param id The ID to be found
       *
       * @return The panel which has the given ID. If the panel was not found,
       *         it returns NULL
       */
      Panel *panelById(int id);
      
      /**
       * Adds a panel to the list of panels and do some initial setup for the
       * panel.
       *  @param pan The panel to be added
       */
      void addPanel(Panel *pan);
      
      /**
       * Deletes a panel
       *  @param pan The panel to be deleted
       */
      void deletePanel(Panel *pan);
      
      /**
       * Recalculates the position of the panels on the screen
       * according to the screen resolution
       */
      void adjustPanelPositions();
      
public slots:
      /**
       * Read the configuration from kpanelsrc
       */
      void readConfig();
      
      /**
       * Receives a notification when the screen size changes
       */
      void slotScreenResized();
      
      /**
       * When a position of a \ref Panel on the screen changes the panel 
       * manager gets notified to upate the config file accordingly
       *  @param pan The panel that was moved on the screen
       */
      void slotPositionChanged(Panel *pan);
      
private:
    PanelList m_panelList;
    bool m_useArgb;
    QString m_theme;
    double m_scaleX;
    double m_scaleY;
    double m_invScaleX;
    double m_invScaleY;
    QRect m_geometry;
};

#endif
