/***************************************************************************
 *   Copyright (C) 2007 by Gustavo Pichorim Boiko                          *
 *   boiko@mandriva.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "selection.h"
#include "panel.h"
#include "panelitem.h"
#include <qpainter.h>
#include <qpixmap.h>
#include <qimage.h>
#include <kdebug.h>
#include <X11/extensions/Xrender.h>

#define MOVE_STEPS 1
#define FADE_STEPS 1
#define INTERVAL 20

Selection::Selection(Panel *parent, const char *name)
 : QObject(parent, name)
{
    m_panel = parent;
    m_item = 0;
    m_anim = 0;
    m_alpha = 0;
    m_fadeStep = 0;
    m_moveStep = 0;
    m_fadeDelta = 0;
    m_moveDelta = 0;
    
    connect(&m_timer, SIGNAL(timeout()), this, SLOT(slotAnimate()));
}


Selection::~Selection()
{
}

void Selection::setColor(const QColor &c)
{
    m_color = c;
    m_panel->repaint(m_rect);
}

void Selection::selectItem(PanelItem *item)
{
    PanelItem *oldItem = m_item;
    
    if (item == m_item)
        return;
        
    m_item = item;
    
    if (oldItem || m_alpha > 0)
    {
        if (m_anim & ANIM_FADEOUT)
        {
            m_anim &= ~ANIM_FADEOUT;
            m_anim |= ANIM_FADEIN;
            m_fadeStep = FADE_STEPS - m_fadeStep;
        }
        m_moveStep = MOVE_STEPS;
        QRect newRect = expand(m_panel->fromBase(m_item->rect()));
        m_moveDelta = (newRect.top() - m_rect.top()) / (float) MOVE_STEPS;
        m_topOrigin = m_rect.top();
        m_rect.setWidth(newRect.width());
        m_rect.setHeight(newRect.height());
        m_anim |= ANIM_MOVE;
    }
    else
    {
        m_fadeDelta = 128. / FADE_STEPS;
        // if fading out, turns to fade in again
        m_fadeStep = FADE_STEPS - m_fadeStep;
        m_anim &= ~ANIM_FADEOUT;
        m_anim |= ANIM_FADEIN;
        m_rect = expand(m_panel->fromBase(item->rect()));
    }
    
    if (!m_timer.isActive())
        m_timer.start(INTERVAL);
}

void Selection::unselect()
{
    if (m_item)
    {
        m_item = 0;
        m_fadeStep = FADE_STEPS - m_fadeStep;
        m_anim &= ~ANIM_FADEIN;
        m_anim |= ANIM_FADEOUT;
        if (!m_timer.isActive())
            m_timer.start(INTERVAL);
    }
}

void Selection::paint(QPixmap &p)
{
    if (!m_item)
        return;
    
    short unsigned int alpha = (unsigned short int) m_alpha;
    if (!alpha)
        return;
            
#if 0
    QPixmap pix(m_rect.size());
    pix.fill(m_color);
    
    XRenderColor clr = { 0, 0, 0, alpha * 0xff};

    XRenderPictureAttributes pa;
    pa.repeat = True;
    Picture pic = XRenderCreatePicture(pix.x11Display(), pix.handle(),
                                     XRenderFindStandardFormat (pix.x11Display(), PictStandardA8),
                                     CPRepeat, &pa);
    XRenderFillRectangle(pix.x11Display(), PictOpSrc, pic,
                         &clr, 0, 0, 1, 1);
    XRenderComposite(pix.x11Display(), PictOpOver, pix.x11RenderHandle(), pic, p.x11RenderHandle(),
                   0,0, 0,0, m_rect.left(),m_rect.top(), m_rect.width(),m_rect.height());
    XRenderFreePicture(pix.x11Display(), pic);
#else
    QPainter paint(&p);
    paint.fillRect(m_rect, m_color);
    if (!m_anim)
        m_item->paint(&paint, m_panel->fromBase(m_item->rect()));
#endif
}

void Selection::slotAnimate()
{
    QRect oldRect = m_rect;
    
    /* Fade in/out handling */
    if ((m_anim & ANIM_FADEIN) || (m_anim & ANIM_FADEOUT))
    {
        if (m_fadeStep)
        {
            if (m_anim & ANIM_FADEIN)
                m_alpha = 128. - (128. / FADE_STEPS) * m_fadeStep;
            else if (m_anim & ANIM_FADEOUT)
                m_alpha = (128. / FADE_STEPS) * m_fadeStep;
                
        }
        else
        {
            if (m_anim & ANIM_FADEIN)
                m_alpha = 128.;
            else if (m_anim & ANIM_FADEOUT)
                m_alpha = 0.;
            m_anim &= ~ANIM_FADEOUT;
            m_anim &= ~ANIM_FADEIN;
        }
    }
    
    /* Move handling */
    if (m_anim & ANIM_MOVE)
    {
        if (m_moveStep)
        {
            m_topOrigin += m_moveDelta;
            m_rect.moveTop((int) m_topOrigin);
        }
        else
        {
            if (m_item)
                m_rect = expand(m_panel->fromBase(m_item->rect()));
            m_anim &= ~ANIM_MOVE;
        }
    }
    
    if (!m_anim)
    {
        m_timer.stop();
    }
    else
    {
        if ((m_anim & ANIM_FADEIN) || (m_anim & ANIM_FADEOUT))
            m_fadeStep--;
        if (m_anim & ANIM_MOVE)
            m_moveStep--;
    }
    m_panel->repaint(expand(m_rect | oldRect));
}

QRect Selection::rect() const
{
    return m_rect;
}

QRect Selection::expand(const QRect &rect)
{
    QRect newRect = rect;
    newRect.moveTop(newRect.top() - 5);
    newRect.moveLeft(newRect.left() - 5);
    newRect.setWidth(newRect.width() + 10);
    newRect.setHeight(newRect.height() + 10);
    return newRect;
}

#include "selection.moc"
